import unittest
import pynasafirms


class TestArea(unittest.TestCase):

    def setUp(self) -> None:
        self.nf = pynasafirms.NasaFirms()

    def test_get_data_availability_all(self):
        self.assertEqual(self.nf.get_data_availability_all()[0].data_id, 'MODIS_NRT')

    def test_get_data_availability_modis_nrt(self):
        self.assertEqual(self.nf.get_data_availability_modis_nrt()[0].data_id, 'MODIS_NRT')

    def test_get_data_availability_modis_sp(self):
        self.assertEqual(self.nf.get_data_availability_modis_sp()[0].data_id, 'MODIS_SP')

    def test_get_data_availability_viirs_noaa20_nrt(self):
        self.assertEqual(self.nf.get_data_availability_viirs_noaa20_nrt()[0].data_id,
                         'VIIRS_NOAA20_NRT')

    def test_get_data_availability_viirs_snpp_sp(self):
        self.assertEqual(self.nf.get_data_availability_viirs_snpp_sp()[0].data_id,
                         'VIIRS_SNPP_SP')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
"""API wrapper for NASA's FIRMS API

This module wraps NASA's FIRMS API for usage within python3. NASA's FIRMS API is used to query data
returned by sensors on NASA's satellites which detect active fires around the globe.

Todo:
    * Better exception handling
    * Better tests
"""
import sys
import os
import csv
from datetime import date
import requests
import detection


class NasaFirms:
    """API wrapper class for the NASA FIRMS API

    Attributes:
        self.host           (str): Base hostname for NASA's FIRMS
        self.country_url    (str): API endpoint path for the 'country' endpoint
        self.area_url       (str): API endpoint path for the 'area' endpoint
        self.data_url       (str): API endpoint path for the 'data_availability' endpoint
        self.map_key        (str): Map key provided by NASA FIRMS which is required to query the API

    """
    def __init__(self, map_key: str = None):
        self.host = 'firms.modaps.eosdis.nasa.gov'
        self.country_url = 'https://{0}/api/country/csv/{1}/{2}/{3}/{4}/{5}'
        self.area_url = 'https://{0}/api/area/csv/{1}/{2}/{3}/{4}/{5}'
        self.data_url = 'https://{0}/api/data_availability/csv/{1}/{2}'
        if map_key:
            self.map_key = map_key
        else:
            try:
                self.map_key = os.environ['FIRMS_MAP_KEY']
            except KeyError:
                print('WARNING: No NASA FIRMS Map Key has been passed to the FIRMS obj or set in '
                      'FIRMS_MAP_KEY environment variable')
                sys.exit(1)

    def csv_to_dict(self,
                    csv_data:   bytes,
                    delimiter:  str = ',') -> list:
        """Converts the bytes data returned from the NASA FIRMS API

        The NASA FIRMS API returns bytes encoded comma or semicolon separated values.
        This function decodes the bytes to a utf-8 string, and then leverages the
        'csv' built-in module to convert the comma/semicolon seoarated values into
        a list of dictionaries, and returns it.

        Args:
            csv_data (bytes): bytes encoded comma/semicolon separated values.
            delimiter (str): default delimiter is ','

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        results = []
        decoded_datum = csv_data.decode('utf-8').splitlines()
        parsed_datum = csv.DictReader(decoded_datum, delimiter=delimiter)
        for result in parsed_datum:
            results.append(result)
        return results

    def get_country_modis_nrt(self,
                              country_code: str,
                              days:         int = 1,
                              start_date:   str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/country' endpoint with the MODIS_NRT sensor

        The country endpoint takes a country code listed on:

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.
        Info on

        Args:
            country_code (str): Country code
            days (int): Number of days to search from start_date
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.country_url.format(self.host,
                                                     self.map_key,
                                                     'MODIS_NRT',
                                                     country_code,
                                                     days,
                                                     start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_country(converted)
        return converted

    def get_country_modis_sp(self,
                             country_code:  str,
                             days:          int = 1,
                             start_date:    str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/country' endpoint with the MODIS_SP sensor

        The country endpoint takes a country code listed on:

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            country_code (str): Country code
            days (int): Number of days to search from start_date
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.country_url.format(self.host,
                                                     self.map_key,
                                                     'MODIS_SP',
                                                     country_code,
                                                     days,
                                                     start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_country(converted)
        return converted

    def get_country_viirs_noaa20_nrt(self,
                                     country_code:  str,
                                     days:          int = 1,
                                     start_date:    str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/country' endpoint with the VIIRS_NOAA20_NRT sensor

        The country endpoint takes a country code listed on:

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            country_code (str): Country code
            days (int): Number of days to search from start_date
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.country_url.format(self.host,
                                                     self.map_key,
                                                     'VIIRS_NOAA20_NRT',
                                                     country_code,
                                                     days,
                                                     start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_country(converted)
        return converted

    def get_country_viirs_snpp_nrt(self,
                                   country_code:    str,
                                   days:            int = 1,
                                   start_date:      str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/country' endpoint with the VIIRS_SNPP_NRT sensor

        The country endpoint takes a country code listed on:

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            country_code (str): Country code
            days (int): Number of days to search from start_date
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.country_url.format(self.host,
                                                     self.map_key,
                                                     'VIIRS_SNPP_NRT',
                                                     country_code,
                                                     days,
                                                     start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_country(converted)
        return converted

    def get_country_viirs_snpp_sp(self,
                                  country_code: str,
                                  days:         int = 1,
                                  start_date:   str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/country' endpoint with the VIIRS_SNPP_SP sensor

        The country endpoint takes a country code listed on:

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            country_code (str): Country code
            days (int): Number of days to search from start_date
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.country_url.format(self.host,
                                                     self.map_key,
                                                     'VIIRS_SNPP_SP',
                                                     country_code,
                                                     days,
                                                     start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_country(converted)
        return converted

    def get_supported_country_codes(self) -> list:
        """Queries the '/countries' to get all the supported countries.

        https://firms.modaps.eosdis.nasa.gov/api/countries/?format=html

        Args:
            No arguments,

        Returns:
            list: List of dictionaries containing the supported country information
        """

        results = []
        datum = requests.get('https://' + self.host + '/api/countries/')
        parsed_resp = self.csv_to_dict(datum.content, delimiter=';')
        for i in parsed_resp:
            results.append(i)
        return results

    def get_area_modis_nrt(self,
                           area:        str,
                           days:        int = 1,
                           start_date:  str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/area' endpoint with the MODIS_NRT sensor

        The area endpoint takes compass directions (north, south, east, west) or 'area coordinates'

        Info on the 'area_coordinates' parameter is available at:
        https://firms.modaps.eosdis.nasa.gov/api/area/

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            area (str): Coordinates or compass direction
            days (int): Number of days to search from start_date (minumum 1, max 10)
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.area_url.format(self.host,
                                                  self.map_key,
                                                  'MODIS_NRT',
                                                  area,
                                                  days,
                                                  start_date))
        converted = self.csv_to_dict(datum.content)
        return converted

    def get_area_modis_sp(self,
                          area:         str,
                          days:         int = 1,
                          start_date:   str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/area' endpoint with the MODIS_SP sensor

        The area endpoint takes compass directions (north, south, east, west) or 'area coordinates'

        Info on the 'area_coordinates' parameter is available at:
        https://firms.modaps.eosdis.nasa.gov/api/area/

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            area (str): Coordinates or compass direction
            days (int): Number of days to search from start_date (minumum 1, max 10)
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.area_url.format(self.host,
                                                  self.map_key,
                                                  'VIIRS_SNPP_SP',
                                                  area,
                                                  days,
                                                  start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_area_viirs_noaa20_nrt(self,
                                  area:         str,
                                  days:         int = 1,
                                  start_date:   str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/area' endpoint with the VIIRS_NOAA20_NRT sensor

        The area endpoint takes compass directions (north, south, east, west) or 'area coordinates'

        Info on the 'area_coordinates' parameter is available at:
        https://firms.modaps.eosdis.nasa.gov/api/area/

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            area (str): Coordinates or compass direction
            days (int): Number of days to search from start_date (minumum 1, max 10)
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.area_url.format(self.host,
                                                  self.map_key,
                                                  'VIIRS_NOAA20_NRT',
                                                  area,
                                                  days,
                                                  start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_area_viirs_snpp_nrt(self,
                                area:       str,
                                days:       int = 1,
                                start_date: str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/area' endpoint with the VIIRS_SNPP_NRT sensor

        The area endpoint takes compass directions (north, south, east, west) or 'area coordinates'

        Info on the 'area_coordinates' parameter is available at:
        https://firms.modaps.eosdis.nasa.gov/api/area/

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            area (str): Coordinates or compass direction
            days (int): Number of days to search from start_date (minumum 1, max 10)
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.area_url.format(self.host,
                                                  self.map_key,
                                                  'VIIRS_SNPP_NRT',
                                                  area,
                                                  days,
                                                  start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_area_viirs_snpp_sp(self,
                               area:        str,
                               days:        int = 1,
                               start_date:  str = date.today().strftime('%Y-%m-%d')) -> list:
        """Queries the '/area' endpoint with the VIIRS_SNPP_SP sensor

        The area endpoint takes compass directions (north, south, east, west) or 'area coordinates'

        Info on the 'area_coordinates' parameter is available at:
        https://firms.modaps.eosdis.nasa.gov/api/area/

        It also takes a day range parameter 'days' and a %Y-%m-%d formatted date to query for data
        on said date.

        Args:
            area (str): Coordinates or compass direction
            days (int): Number of days to search from start_date (minumum 1, max 10)
            start_date (str): Date in %Y-%m-%d format

        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.area_url.format(self.host,
                                                  self.map_key,
                                                  'VIIRS_SNPP_SP',
                                                  area,
                                                  days,
                                                  start_date))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_data_availability_all(self) -> list:
        """Queries the '/data_availability' endpoint with all sensors

        Gets the availability of 'date' data for the specified sensor.

        Info on the 'data_availability' endpoint is available at:
        https://firms.modaps.eosdis.nasa.gov/api/data_availability/

        Args:
            No arguments.
        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.data_url.format(self.host, self.map_key, 'ALL'))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_data_availability_modis_nrt(self) -> list:
        """Queries the '/data_availability' endpoint with the MODIS_NRT sensor

        Gets the availability of 'date' data for the specified sensor.

        Info on the 'data_availability' endpoint is available at:
        https://firms.modaps.eosdis.nasa.gov/api/data_availability/

        Args:
            No arguments.
        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.data_url.format(self.host, self.map_key, 'MODIS_NRT'))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_data_availability_modis_sp(self) -> list:
        """Queries the '/data_availability' endpoint with the MODIS_SP sensor

        Gets the availability of 'date' data for the specified sensor.

        Info on the 'data_availability' endpoint is available at:
        https://firms.modaps.eosdis.nasa.gov/api/data_availability/

        Args:
            No arguments.
        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.data_url.format(self.host, self.map_key, 'MODIS_SP'))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_data_availability_viirs_noaa20_nrt(self) -> list:
        """Queries the '/data_availability' endpoint with the VIIRS_NOAA20_NRT sensor

        Gets the availability of 'date' data for the specified sensor.

        Info on the 'data_availability' endpoint is available at:
        https://firms.modaps.eosdis.nasa.gov/api/data_availability/

        Args:
            No arguments.
        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.data_url.format(self.host, self.map_key, 'VIIRS_NOAA20_NRT'))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def get_data_availability_viirs_snpp_sp(self) -> list:
        """Queries the '/data_availability' endpoint with the VIIRS_SNPP_SP sensor

        Gets the availability of 'date' data for the specified sensor.

        Info on the 'data_availability' endpoint is available at:
        https://firms.modaps.eosdis.nasa.gov/api/data_availability/

        Args:
            No arguments.
        Returns:
            list: List of dictionaries containing the decoded and converted api response
        """
        datum = requests.get(self.data_url.format(self.host, self.map_key, 'VIIRS_SNPP_SP'))
        converted = self.csv_to_dict(datum.content)
        converted = self.response_to_area(converted)
        return converted

    def response_to_area(self, response: list) -> list:
        """Converts a response from the 'area' endpoint to an Area class.

        Args:
            response (list): A list of dictionaries containing /area endpoint repsonses
        Returns:
            list: List of Area objects converted from the api response
        """
        converted = []
        if len(response) > 0:
            for resp in response:
                area = detection.detection.Area()
                for key in resp:
                    setattr(area, key, resp[key])
                converted.append(area)
        return converted

    def response_to_country(self, response: list) -> list:
        """Converts a response from the 'area' endpoint to an Area class.

        Args:
            response (list): A list of dictionaries containing /area endpoint repsonses
        Returns:
            list: List of Area objects converted from the api response
        """
        converted = []
        if len(response) > 0:
            for resp in response:
                area = detection.detection.Country()
                for key in resp:
                    setattr(area, key, resp[key])
                converted.append(area)
        return converted



class Area:
    def __init__(self,
                 latitude:      str = None,
                 longitude:     str = None,
                 bright_ti4:    str = None,
                 scan:          str = None,
                 track:         str = None,
                 acq_date:      str = None,
                 acq_time:      str = None,
                 satellite:     str = None,
                 instrument:    str = None,
                 confidence:    str = None,
                 version:       str = None,
                 bright_ti5:    str = None,
                 frp:           str = None,
                 daynight:      str = None,
                 brightness:    str = None,
                 bright_t31:    str = None):
        self.latitude = latitude
        self.longitude = longitude
        self.bright_ti4 = bright_ti4
        self.scan = scan
        self.track = track
        self.acq_date = acq_date
        self.acq_time = acq_time
        self.satellite = satellite
        self.instrument = instrument
        self.confidence = confidence
        self.version = version
        self.bright_ti5 = bright_ti5
        self.frp = frp
        self.daynight = daynight
        self.brightness = brightness
        self.bright_t31 = bright_t31

    def __str__(self):
        return f"Area(latitude={self.latitude}, " \
               f"longitude={self.longitude}, " \
               f"bright_ti4={self.bright_ti4}, " \
               f"scan={self.scan}, " \
               f"track={self.track}, " \
               f"acq_date={self.acq_date}, " \
               f"acq_time={self.acq_time}, " \
               f"satellite={self.satellite}, " \
               f"instrument={self.instrument}, " \
               f"confidence={self.version}, " \
               f"bright_ti5={self.bright_ti5}, " \
               f"frp={self.frp}, " \
               f"daynight={self.daynight}, " \
               f"brightness={self.brightness}, " \
               f"bright_t31={self.bright_t31}"

    def __repr__(self):
        return f"Area(latitude={self.latitude}, " \
               f"longitude={self.longitude}, " \
               f"bright_ti4={self.bright_ti4}, " \
               f"scan={self.scan}, " \
               f"track={self.track}, " \
               f"acq_date={self.acq_date}, " \
               f"acq_time={self.acq_time}, " \
               f"satellite={self.satellite}, " \
               f"instrument={self.instrument}, " \
               f"confidence={self.version}, " \
               f"bright_ti5={self.bright_ti5}, " \
               f"frp={self.frp}, " \
               f"daynight={self.daynight}, " \
               f"brightness={self.brightness}, " \
               f"bright_t31={self.bright_t31}"


class Country:
    def __init__(self,
                 country_id:    str = None,
                 latitude:      str = None,
                 longitude:     str = None,
                 bright_ti4:    str = None,
                 scan:          str = None,
                 track:         str = None,
                 acq_date:      str = None,
                 acq_time:      str = None,
                 satellite:     str = None,
                 instrument:    str = None,
                 confidence:    str = None,
                 version:       str = None,
                 bright_ti5:    str = None,
                 frp:           str = None,
                 daynight:      str = None,
                 brightness:    str = None,
                 bright_t31:    str = None):
        self.country_id = country_id
        self.latitude = latitude
        self.longitude = longitude
        self.bright_ti4 = bright_ti4
        self.scan = scan
        self.track = track
        self.acq_date = acq_date
        self.acq_time = acq_time
        self.satellite = satellite
        self.instrument = instrument
        self.confidence = confidence
        self.version = version
        self.bright_ti5 = bright_ti5
        self.frp = frp
        self.daynight = daynight
        self.brightness = brightness
        self.bright_t31 = bright_t31

    def __str__(self):
        return f"Country(country_id={self.country_id}, " \
               f"latitude={self.latitude}, " \
               f"longitude={self.longitude}, " \
               f"bright_ti4={self.bright_ti4}, " \
               f"scan={self.scan}, " \
               f"track={self.track}, " \
               f"acq_date={self.acq_date}, " \
               f"acq_time={self.acq_time}, " \
               f"satellite={self.satellite}, " \
               f"instrument={self.instrument}, " \
               f"confidence={self.confidence}, " \
               f"version={self.version}, " \
               f"bright_ti5={self.bright_ti5}, " \
               f"frp={self.frp}, " \
               f"daynight={self.daynight}, " \
               f"brightness={self.brightness}, " \
               f"bright_t31={self.bright_t31})"

    def __repr__(self):
        return f"Country(country_id={self.country_id}, " \
               f"latitude={self.latitude}, " \
               f"longitude={self.longitude}, " \
               f"bright_ti4={self.bright_ti4}, " \
               f"scan={self.scan}, " \
               f"track={self.track}, " \
               f"acq_date={self.acq_date}, " \
               f"acq_time={self.acq_time}, " \
               f"satellite={self.satellite}, " \
               f"instrument={self.instrument}, " \
               f"confidence={self.confidence}, " \
               f"version={self.version}, " \
               f"bright_ti5={self.bright_ti5}, " \
               f"frp={self.frp}, " \
               f"daynight={self.daynight}, " \
               f"brightness={self.brightness}, " \
               f"bright_t31={self.bright_t31})"



from distutils.core import setup

setup(name='pynasafirms',
      version='0.0.1',
      description='API Wrapper for NASA\'s FIRMS',
      author='PlantDaddy',
      author_email='CqP5TZ77NYBf5uQt@protonmail.com',
      url='https://github.com/PlantDaddy/pyNASAFIRMS',
      packages=['distutils', 'distutils.command'],
      install_requires=['requests']
     )



import unittest
import pynasafirms


class TestCountry(unittest.TestCase):

    def setUp(self) -> None:
        self.nf = pynasafirms.NasaFirms()

    def test_country_modis_nrt(self):
        self.assertEqual(self.nf.get_country_modis_nrt('UKR', start_date='2022-04-04')[0].country_id,
                         'UKR')

    def test_country_modis_sp(self):
        self.assertEqual(self.nf.get_country_modis_sp('UKR', start_date='2022-04-04'), [])

    def test_country_viirs_noaa20_nrt(self):
        self.assertEqual(self.nf.get_country_viirs_noaa20_nrt('UKR', start_date='2022-04-04')[0].country_id, 'UKR')

    def test_country_viirs_snpp_nrt(self):
        self.assertEqual(self.nf.get_country_viirs_snpp_nrt('UKR', start_date='2022-04-04')[0].country_id,
                         'UKR')

    def test_country_viirs_snpp_sp(self):
        self.assertEqual(self.nf.get_country_viirs_snpp_sp('UKR', start_date='2022-04-04'), [])

    def test_get_supported_countries(self):
        self.assertEqual(self.nf.get_supported_country_codes()[0]['id'], '1')


if __name__ == '__main__':
    unittest.main()
